/* ************************************************************************** *
 *      Copyright © IPSL/LSCE, XMLIOServer, Avril 2010 - Octobre 2011         *
 * ************************************************************************** */

#ifndef __INETCDF4_HPP__
#define __INETCDF4_HPP__

/**
 * \file    inetcdf4.hpp
 * \brief   Lecture des fichiers de données au format netCDF-4 (entête).
 * \author  Hervé Ozdoba
 * \version 0.4
 * \date    15 Juin 2011
 */

#ifndef __XIOS_NO_EXTERN

// M(essage) P(assing) I(nterface) headers
#include <mpi.h>

// NetCDF headers
#define MPI_INCLUDED
#include <netcdf.h>

#endif // __XIOS_NO_EXTERN

// XMLIOServer headers
#include "xmlioserver_spl.hpp"

// ////////////////////////////// Déclarations ///////////////////////////// //

namespace xmlioserver
{
   /// \brief Espace de nommage permettant de traiter les Entrées/Sorties sous divers formats.
   namespace io
   {
      /**
       * \class CINetCDF4
       * \brief Classe permettant de lire les fichiers de données au format netCDF-4.
       */
      class CINetCDF4
      {
         public : // Définition de type
            
            //! \brief Redéfinition de std::vector<std::string> pour simplification d'écriture.
            typedef std::vector<std::string> CNetCDF4Path;
            
         public : // Constructeurs
         
            //! \brief Constructeur simple.
            CINetCDF4(void); // NIY
            
            /*!
             * \brief Constructeur à partir d'un nom de fichier au format netCDF4.
             * \param _filename : un nom de fichier au format netCDF4.
             */
            explicit CINetCDF4(const std::string & _filename);
            
            /*!
             * \brief Constructeur par copie .
             * \param _inetcdf4 : référence de l'objet à copier.
             */
            CINetCDF4(const CINetCDF4 & _inetcdf4);// NIY
            
            /*!
             * \brief Constructeur par copie.
             * \param _inetcdf4_ptr : le pointeur sur l'objet à copier.
             */
            CINetCDF4(const CINetCDF4 * const _inetcdf4_ptr); // NIY
            
         public : // Destructeur
         
            //! \brief Destructeur de l'objet (le fichier est clos lors de son appel).
            ~CINetCDF4(void);

         protected : // Accesseurs protégés
         
            /*!
             * \brief Retourne l'identifiant du groupe courant 
             * (le chemin est gardée en mémoire dans la variable path).
             * \return  l'identifiant du groupe courant
             * \see path
             */
            int getCurrentGroup(void);
            
            /*!
             * \brief Retourne l'identifiant du groupe  dont le chemin est passé en argument.
             * \param _path : Un chemin dans la structure de donnée netCDF.
             * \return l'identifiant du groupe.
             *
             * <i> Exemple d'utilisation: </i>
             * \code
             * //> Retourne l'identifiant d'un group.
             * CINetCDF4 ncinput(std::string("mon_fichier.nc"));
             * CNetCDF4Path path;
             * path.push_back("mes_donnees"); path.push_back("mes_tableaux");
             * int groupid = ncinput.getGroup(path);
             * \endcode
             */
            int getGroup(const CNetCDF4Path & _path);
            
            /*!
             * \brief Retourne l'identifiant d'une variable dans le groupe courant.
             * \param _varname : Le nom d'une variable dans le fichier de données.
             * \return l'identifiant de la variable.
             */
            int getVariable(const std::string & _varname);
            
            /*!
             * \brief Retourne l'identifiant d'une dimension dans le groupe courant.
             * \param _dimname : Le nom d'une dimension dans le  fichier de données.
             * \return l'identifiant de la dimension.
             */
            int getDimension(const std::string & _dimname);
            
            /*!
             * \brief Retourne les dimensions d'une variable.
             * \param _varname : Le nom d'une variable dans le  fichier de données.
             * \return Un vecteur des dimensions associées à la variables
             */
            std::vector<std::size_t> getDimensions(const std::string & _varname);
            
            /*!
             * \brief Retourne l'identifiant de la dimension non limité dans le fichier.
             * \return l'identifiant de la dimension non limité.
             */
            int getUnlimitedDimension(void);
            
            /*!
             * \brief Indique si une variable existe dans le groupe courant.
             * \param _varname : Le nom d'une variable dans le  fichier de données.
             * \return  <b>true</b> si la variable existe, <b>false</b> sinon.
             */
            bool varExist(const std::string & _varname);


         private : // Vérification des erreurs NetCDF

            /*!
             * \brief Vérifie le code d'erreur retourné lors de l'appel d'une fonction
             * dans la bibliothèque netCDF afin de détecter les erreurs.
             * \param _status : le code d'erreur retourné par l'appel d'une fonction netCDF.
             * \remarks Tous les appels de fonction dans la bibliothèque netCDF doivent être
             * pris en charge par cette méthode statique.
             * 
             * <i> Exemple d'utilisation: </i>
             * \code
             * //> Retourne l'identifiant d'une variable netCDF dont le nom est 'varname' dans le groupe grpid 
             * CheckError(nc_inq_varid (grpid, varname, varid));
             * \endcode
             */
            static void CheckError(int _status);

         public : // Propriétés privées
         
            CONetCDF4Path path; /*!< \brief Chemin vers le groupe de données en cours de lecture. */
            int ncidp;          /*!< \brief Identifiant de fichiers netCDF. */
         
      }; // class CINetCDF4
   } // namespace io

} // namespace xmlioserver

#endif	// __INETCDF4_HPP__
